#pragma once

// Result code
typedef enum
{
    // Operation succeeded.
    IPI_OK = 0,
    // Some error has occurred.
    IPI_GENERAL_ERROR = 1,
    // Invalid parameter value in a call to function.
    IPI_INVALID_PARAM = 2,
    // Not enough memory to complete operation.
    IPI_OUT_OF_MEMORY = 3,
    // Connection to an application has been broken. Should call iPi_Disconnect on this one and establish a new connection if needed.
    IPI_CONNECTION_BROKEN = 4,
    // Application executable file was not found or other problem when starting application.
    IPI_CANNOT_START_APP = 5,
    // Application is not running or does not accept connections. For example, its an old version of the application which does not support connections. Or limit on connections is reached.
    IPI_CANNOT_CONNECT = 6,
    // Invalid data read from a connection.
    IPI_INVALID_DATA = 7,
} iPi_Result;

// Callback function for handling application events.
// PARAMETERS
//     hConnection: Handle of a connection the event has occurred on.
//     jsonEvent: String with JSON representation of event.
// REMARKS
// Structure of event is:
//     {
//         "event": "event name",
//         "arguments": { /* optional */
//                          "arg_name1": arg_value1,
//                          ...
//                          "arg_nameN": arg_valueN
//                      }
//     }
typedef void (WINAPI *iPi_EventCallback)(HANDLE hConnection, LPWSTR jsonEvent);
