﻿// TestClient.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"
#include "json.h"
#include "util.h"
#include <fstream>
#include <windows.h>


CRITICAL_SECTION lockAwaitedEvent;
HANDLE awaitedEventReceived;
std::string awaitedEventName;
Json::Value awaitedEvent;


void ParseAppEventAndCheckIfAwaited(LPWSTR jsonEvent)
{
	std::wstring jsonEventStr(jsonEvent);
	Json::Value appEvent = StringToJson(jsonEventStr);
	std::string eventName = appEvent["event"].asString();
	if (eventName == awaitedEventName)
	{
		awaitedEventName.clear();
		awaitedEvent = appEvent;
		SetEvent(awaitedEventReceived);
	}
}

void WINAPI AppEventCallback(HANDLE hConnection, LPWSTR jsonEvent)
{
	__try
	{
		wprintf(L"App event: %s\r\n", jsonEvent);

		EnterCriticalSection(&lockAwaitedEvent);
		__try
		{
			if (!awaitedEventName.empty())
			{
				ParseAppEventAndCheckIfAwaited(jsonEvent);
			}
		}
		__finally
		{
			LeaveCriticalSection(&lockAwaitedEvent);
		}
	}
	__finally
	{
		iPi_Free(jsonEvent);
	}
}

void PrepareWaitForAppEvent(LPCSTR eventName)
{
	EnterCriticalSection(&lockAwaitedEvent);
	__try
	{
		awaitedEventName = eventName;
		awaitedEvent.clear();
		ResetEvent(awaitedEventReceived);
	}
	__finally
	{
		LeaveCriticalSection(&lockAwaitedEvent);
	}
}

BOOL WaitForAppEvent(DWORD timeoutMilliseconds)
{
	DWORD waitResult = WaitForSingleObject(awaitedEventReceived, timeoutMilliseconds);
	return waitResult == WAIT_OBJECT_0;
}


std::wstring ReadCommandFromFile(std::string fileName, std::string fileExtension);
std::string GetTestFilePath(LPCWSTR fileName);
std::string GetTestVideoPath();
Json::Value MakeArgumentsForRecording(const Json::Value& refreshResponse);

void RunSequenceForRecorder(HANDLE hConnection)
{
	Json::Value command;
	Json::Value response;
	std::string videoPath = GetTestVideoPath();
	BOOL success;

	command.clear();
	command["command"] = "get-app-info";
	ExecuteCommand(hConnection, command);

	command.clear();
	command["command"] = "open-home-screen";
	ExecuteCommand(hConnection, command);

	Sleep(3000);

	command.clear();
	command["command"] = "refresh-device-list";
	success = ExecuteCommand(hConnection, command, &response);

	if (success && response["success"].asBool())
	{
		Json::Value arguments = MakeArgumentsForRecording(response);


		command.clear();
		command["command"] = "open-recorder";
		command["arguments"] = arguments;
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "get-recorder-props";
		success = ExecuteCommand(hConnection, command, &response);

		if (success)
		{
			command.clear();
			command["command"] = "set-recorder-props";
			command["arguments"]["videoMode"] = response["result"]["videoModeList"][1];
			command["arguments"]["recordingStartDelay"] = 0;
			command["arguments"]["colorCompression"] = "jpeg";
			command["arguments"]["hideBackgroundWhenRecording"] = true;
			ExecuteCommand(hConnection, command);
		}

		PrepareWaitForAppEvent("background-evaluation-stopped");
		command.clear();
		command["command"] = "evaluate-background";
		command["arguments"]["startDelay"] = 1;
		success = ExecuteCommand(hConnection, command, &response);
		if (success && response["success"].asBool())
			WaitForAppEvent(INFINITE);

		command.clear();
		command["command"] = "go-to-screen";
		command["arguments"]["screen"] = "record";
		ExecuteCommand(hConnection, command);

		Sleep(1000);

		command.clear();
		command["command"] = "get-devices-state";
		ExecuteCommand(hConnection, command);

		PrepareWaitForAppEvent("recording-stopped");
		command.clear();
		command["command"] = "start-recording";
		command["arguments"]["filePath"] = videoPath;
		command["arguments"]["startDelay"] = 1;
		success = ExecuteCommand(hConnection, command, &response);
		if (success && response["success"].asBool())
		{
			success = WaitForAppEvent(5000);
			if (!success)
			{
				command.clear();
				command["command"] = "stop-recording";
				ExecuteCommand(hConnection, command);
				WaitForAppEvent(3000);
			}
		}
		else
		{
			// Sometimes recording does not start because fps is lower just after switch to recording tab
			// We'll solve this issue in future versions
			Sleep(3000);
			success = ExecuteCommand(hConnection, command, &response);
			if (success && response["success"].asBool())
			{
				success = WaitForAppEvent(5000);
				if (!success)
				{
					command.clear();
					command["command"] = "stop-recording";
					ExecuteCommand(hConnection, command);
					WaitForAppEvent(INFINITE);
				}
			}
		}

		PrepareWaitForAppEvent("playing-stopped");
		command.clear();
		command["command"] = "open-player";

		command["arguments"]["filePath"] = videoPath;
		success = ExecuteCommand(hConnection, command, &response);

		command.clear();
		command["command"] = "set-player-props";

		command["arguments"]["screensInRow"] = 2;
		command["arguments"]["hideBackground"] = true;
		success = ExecuteCommand(hConnection, command, &response);

		command.clear();
		command["command"] = "play-forward";

		success = ExecuteCommand(hConnection, command, &response);

		if (success && response["success"].asBool())
			WaitForAppEvent(INFINITE);


		Sleep(1000);

		command.clear();
		command["command"] = "frame-forward";

		success = ExecuteCommand(hConnection, command, &response);

		PrepareWaitForAppEvent("playing-stopped");

		command.clear();
		command["command"] = "play-backward";

		success = ExecuteCommand(hConnection, command, &response);
		if (success && response["success"].asBool())
			WaitForAppEvent(INFINITE);

		command.clear();
		command["command"] = "frame-backward";

		success = ExecuteCommand(hConnection, command, &response);

		command.clear();
		command["command"] = "frame-select";
		command["arguments"]["frameIndex"] = 400;

		success = ExecuteCommand(hConnection, command, &response);

		command.clear();
		command["command"] = "go-to-frame";
		command["arguments"]["frameIndex"] = 100;

		success = ExecuteCommand(hConnection, command, &response);

		Sleep(4000);
	}

	command.clear();
	command["command"] = "exit";
	ExecuteCommand(hConnection, command);
}

void RunSequenceForMocapStudio(HANDLE hConnection)
{
	Json::Value command;
	Json::Value response;
	BOOL success;

	command.clear();
	command["command"] = "get-app-info";
	ExecuteCommand(hConnection, command);

	command.clear();
	command["command"] = "create-project";
	command["arguments"]["videoFilePath"] = GetTestVideoPath();
	command["arguments"]["projectType"] = "action";

	Json::Value actor;
	actor.clear();
	actor["gender"] = "male";
	actor["height"] = 1.8;

	command["arguments"]["actors"].append(actor);

	success = ExecuteCommand(hConnection, command, &response);

	if (success && response["success"].asBool())
	{

		command.clear();
		command["command"] = "get-project-info";
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "get-view-props";
		ExecuteCommand(hConnection, command, &response);

		command.clear();
		command["command"] = "set-view-props";
		command["arguments"]["skin"] = false;
		command["arguments"]["skinOpacity"] = 0.5;
		command["arguments"]["bones"] = true;
		command["arguments"]["viewportBackgroundColor"] = "CornflowerBlue";
		command["arguments"]["cameras"] = false;
		command["arguments"]["currentCamera"] = 2;
		command["arguments"]["groundPlane"] = false;

		command["arguments"]["currentActor"] = 0;
		command["arguments"]["trajectories"] = false;
		command["arguments"]["skeletonStrobe"] = false;
		command["arguments"]["extraSkeletonStrobe"] = false;
		command["arguments"]["collisionObjects"] = false;
		command["arguments"]["massObjects"] = false;
		command["arguments"]["poseMismatch"] = false;

		command["arguments"]["video"] = true;
		command["arguments"]["videoOpacity"] = 0.8;
		command["arguments"]["videoThumbnails"] = false;
		command["arguments"]["lightSource"] = true;

		ExecuteCommand(hConnection, command, &response);

		command.clear();
		command["command"] = "get-timeline-props";
		ExecuteCommand(hConnection, command, &response);

		int framesCount = response["result"]["timeline"]["framesCount"].asInt();

		command.clear();
		command["command"] = "set-timeline-props";
		command["arguments"]["regionOfInterest"]["beginFrame"] = 10;
		command["arguments"]["regionOfInterest"]["framesCount"] = framesCount / 20;
		Json::Value take;
		take.clear();
		take["beginFrame"] = 10;
		take["framesCount"] = framesCount / 5;
		take["description"] = "Take 02";
		command["arguments"]["takes"].append(take);
		take["beginFrame"] = 3 * framesCount / 5;
		take["framesCount"] = framesCount / 5;
		take["description"] = "Take 01";
		command["arguments"]["takes"].append(take);
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "set-actor-props";
		command["arguments"]["gender"] = "male";
		command["arguments"]["height"] = 1.99f;
		command["arguments"]["dimensions"]["headHeight"] = 0.186f;
		command["arguments"]["dimensions"]["shouldersWidth"] = 0.494f;
		command["arguments"]["dimensions"]["armLength"] = 0.773f;
		command["arguments"]["dimensions"]["forearmLength"] = 0.245f;
		command["arguments"]["dimensions"]["legLength"] = 0.96f;
		command["arguments"]["dimensions"]["lowerlegLength"] = 0.47f;
		command["arguments"]["dimensions"]["footLength"] = 0.25f;
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "set-actor-props";
		command["arguments"]["gender"] = "male";
		command["arguments"]["dimensions"]["headHeight"] = 0.235f;
		command["arguments"]["dimensions"]["shouldersWidth"] = 0.375f;
		command["arguments"]["dimensions"]["armLength"] = 0.62f;
		command["arguments"]["dimensions"]["legLength"] = 0.96f;
		command["arguments"]["dimensions"]["lowerlegLength"] = 0.50f;
		command["arguments"]["dimensions"]["forearmLength"] = 0.3f;
		command["arguments"]["dimensions"]["footLength"] = 0.32f;
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "get-timeline-props";
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "set-tracking-props";
		ExecuteCommand(hConnection, command, &response);

		command["arguments"]["enableFootTracking"] = true;
		command["arguments"]["enableGroundCollisions"] = false;
		command["arguments"]["enableHeadTracking"] = false;
		command["arguments"]["shouldersTrackingMode"] = 1;
		command["arguments"]["spineTrackingMode"] = 2;
		command["arguments"]["trackingResolution"] = 0;
		command["arguments"]["jitterRemovalOptions"].append(5);
		command["arguments"]["jitterRemovalOptions"].append(4);
		command["arguments"]["jitterRemovalOptions"].append(3);
		command["arguments"]["jitterRemovalOptions"].append(2);
		command["arguments"]["jitterRemovalOptions"].append(1);
		command["arguments"]["jitterRemovalOptions"].append(0);
		command["arguments"]["trajectoryFilter"] = 3;
		ExecuteCommand(hConnection, command, &response);

		command.clear();
		command["command"] = "get-tracking-props";
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "go-to-frame";
		command["arguments"]["frameIndex"] = 10;
		ExecuteCommand(hConnection, command);

		PrepareWaitForAppEvent("refit-pose-finished");

		command.clear();
		command["command"] = "refit-pose";
		success = ExecuteCommand(hConnection, command, &response);
		if (success && response["success"].asBool())
			WaitForAppEvent(INFINITE);

		PrepareWaitForAppEvent("detect-actor-colors-finished");

		command.clear();
		command["command"] = "detect-actor-colors";
		command["arguments"]["handsColorSameAsFace"] = false;
		success = ExecuteCommand(hConnection, command, &response);
		if (success && response["success"].asBool())
			WaitForAppEvent(INFINITE);

		command.clear();
		command["command"] = "delete-pose";
		success = ExecuteCommand(hConnection, command, &response);
		Sleep(1000);

		PrepareWaitForAppEvent("refit-pose-finished");

		command.clear();
		command["command"] = "refit-pose";
		success = ExecuteCommand(hConnection, command, &response);
		if (success && response["success"].asBool())
			WaitForAppEvent(INFINITE);

		PrepareWaitForAppEvent("tracking-operation-finished");

		command.clear();
		command["command"] = "track-forward";
		success = ExecuteCommand(hConnection, command, &response);
		if (success && response["success"].asBool())
		{
			WaitForAppEvent(INFINITE);
		}

		command.clear();
		command["command"] = "refine-backward";
		success = ExecuteCommand(hConnection, command, &response);
		Sleep(5000);

		command.clear();
		command["command"] = "pause-tracking";
		success = ExecuteCommand(hConnection, command, &response);

		PrepareWaitForAppEvent("tracking-operation-finished");

		command.clear();
		command["command"] = "remove-jitter";
		success = ExecuteCommand(hConnection, command, &response);
		if (success && response["success"].asBool())
			WaitForAppEvent(INFINITE);

		command.clear();
		command["command"] = "play-backward";
		ExecuteCommand(hConnection, command);
		Sleep(2000);

		command.clear();
		command["command"] = "pause-play";
		ExecuteCommand(hConnection, command);
		Sleep(1000);

		command.clear();
		command["command"] = "frame-forward";
		ExecuteCommand(hConnection, command);
		Sleep(1000);

		command.clear();
		command["command"] = "frame-backward";
		ExecuteCommand(hConnection, command);
		Sleep(1000);

		command.clear();
		command["command"] = "play-forward";
		ExecuteCommand(hConnection, command);
		Sleep(1000);

		command.clear();
		command["command"] = "pause-play";
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "get-export-props";
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "load-target-character";
		command["arguments"]["rigName"] = "MotionBuilder";
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "set-export-props";
		command["arguments"]["advancedMotionTransfer"] = true;
		command["arguments"]["exportTPoseInFirstFrame"] = true;
		command["arguments"]["recenterCoordinateSystemOnCharacter"] = true;
		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "export-animation";
		command["arguments"]["filePath"] = GetTestFilePath(L"test.fbx");
		command["arguments"]["fbxIsBinary"] = true;
		command["arguments"]["fbxVersion"] = "2014";

		ExecuteCommand(hConnection, command);

		command.clear();
		command["command"] = "close-project";
		ExecuteCommand(hConnection, command);
	}

	command.clear();
	command["command"] = "exit";
	ExecuteCommand(hConnection, command);
}

int _tmain(int argc, _TCHAR* argv[])
{
	ZeroMemory(&lockAwaitedEvent, sizeof(CRITICAL_SECTION));
	InitializeCriticalSection(&lockAwaitedEvent);
	awaitedEventReceived = CreateEvent(NULL, TRUE, FALSE, NULL);

	iPi_Result result;
	HANDLE hConnection;

	result = iPi_ConnectToRecorder(NULL, AppEventCallback, &hConnection);
	if (result != IPI_OK)
	{
		printf("Error connecting: %d\r\n", result);
		return 0;
	}

	printf("Connected\r\n");

	try
	{
		RunSequenceForRecorder(hConnection);
	}
	catch (const iPi_Result& result)
	{
		if (result != IPI_CONNECTION_BROKEN)
			throw;
	}

	result = iPi_ConnectToMocapStudio(NULL, AppEventCallback, &hConnection);
	if (result != IPI_OK)
	{
		printf("Error connecting: %d\r\n", result);
		return 0;
	}

	printf("Connected\r\n");

	try
	{
		RunSequenceForMocapStudio(hConnection);
	}
	catch (const iPi_Result& result)
	{
		if (result != IPI_CONNECTION_BROKEN)
			throw;
	}

	result = iPi_Disconnect(hConnection);
	if (result == IPI_OK)
	{
		printf("Disconnected\r\n");
	}
	else
	{
		printf("Error disconnecting: %d\r\n", result);
	}

	DeleteCriticalSection(&lockAwaitedEvent);
	CloseHandle(awaitedEventReceived);
}

std::wstring ReadCommandFromFile(std::string scriptFilePath, std::string fileExtension)
{
	std::ifstream file(scriptFilePath);
	std::stringstream buffer;
	buffer << file.rdbuf();
	std::string scriptStr = buffer.str();
	std::wstring scriptWstr(scriptStr.begin(), scriptStr.end());
	return scriptWstr;
}

std::string GetTestVideoPath()
{
	return GetTestFilePath(L"test.iPiVideo");
}

std::string GetTestFilePath(LPCWSTR fileName)
{
	WCHAR currDir[MAX_PATH + 1];
	GetCurrentDirectoryW(MAX_PATH + 1, currDir);
	std::wstring result = currDir;
	result += L"\\";
	result += fileName;
	return Utf16ToUtf8(result);
}

Json::Value MakeArgumentsForRecording(const Json::Value& refreshResponse)
{
	Json::Value devices = refreshResponse["result"];
	Json::Value selectedDevices;
	for (Json::Value::ArrayIndex i = 0; i < devices.size(); i++)
	{
		Json::Value device = devices[i];
		std::string kind = device["kind"].asString();
		if (kind == "camera")
		{
			std::string id = device["id"].asString();
			selectedDevices.append(id);
		}
	}
	Json::Value arguments;
	arguments["selectedDevices"] = selectedDevices;
	return arguments;
}
