#include "stdafx.h"
#include "util.h"
#include <sstream>
#include <codecvt>


std::string Utf16ToUtf8(const std::wstring& s)
{
    std::wstring_convert<std::codecvt_utf8_utf16<WCHAR>> convert;
    return convert.to_bytes(s);
}

std::wstring Utf8ToUtf16(const std::string& s)
{
    std::wstring_convert<std::codecvt_utf8_utf16<WCHAR>> convert;
    return convert.from_bytes(s);
}

Json::Value StringToJson(const std::wstring& s)
{
    std::string utf8 = Utf16ToUtf8(s);
    Json::CharReaderBuilder readerBuilder;
    readerBuilder["collectComments"] = false;
    readerBuilder["rejectDupKeys"] = true;
    readerBuilder["strictRoot"] = true;
    readerBuilder["allowSpecialFloats"] = true;
    readerBuilder["failIfExtra"] = true;
    std::stringstream stream(utf8);
    Json::Value json;
    std::string errors;
    bool parsed = Json::parseFromStream(readerBuilder, stream, &json, &errors);
    return parsed ? json : Json::Value::null;
}

std::wstring JsonToString(const Json::Value& json)
{
    Json::StreamWriterBuilder writerBuilder;
    writerBuilder["commentStyle"] = "None";
    writerBuilder["indentation"] = Json::Value::null;
    writerBuilder["useSpecialFloats"] = true;
    std::string utf8 = Json::writeString(writerBuilder, json);
    return Utf8ToUtf16(utf8);
}

bool ExecuteCommand(HANDLE hConnection, LPCWSTR command, Json::Value* responseJson)
{
    LPWSTR response;
    iPi_Result result;
    wprintf(L"Executing: %s\r\n", command);
    result = iPi_ExecuteCommand(hConnection, command, &response);
    if (result == IPI_OK)
    {
        wprintf(L"Received: %s\r\n", response);

        std::wstring responseStr(response);
        Json::Value _responseJson = StringToJson(responseStr);
        iPi_Free(response);

        if (_responseJson.isNull())
        {
            return false;
        }
        else if (responseJson != nullptr)
        {
            *responseJson = _responseJson;
        }

        return true;
    }
    else
    {
        printf("Error executing: %d\r\n", result);
        if (result == IPI_CONNECTION_BROKEN)
            throw result;
        else
            return false;
    }
}

bool ExecuteCommand(HANDLE hConnection, LPCWSTR command)
{
    return ExecuteCommand(hConnection, command, nullptr);
}

bool ExecuteCommand(HANDLE hConnection, const Json::Value& command, Json::Value* response)
{
    std::wstring commandStr = JsonToString(command);
    return ExecuteCommand(hConnection, commandStr.c_str(), response);
}

bool ExecuteCommand(HANDLE hConnection, const Json::Value& command)
{
    return ExecuteCommand(hConnection, command, nullptr);
}
